// only one switcher can be active
var __PanelSwitcher_currentSwitcher = null;


// constructor
function PanelSwitcher( frontPanel, backPanel, frontShadow ) {
    this.frontPanel  = frontPanel;
    this.backPanel   = backPanel;
    this.frontShadow = frontShadow;

    var z_front  = frontPanel.style.zIndex;
    var z_back   = backPanel.style.zIndex;
    if ( !z_front ) z_front = 200;
    if ( !z_back ) z_back = 100;
    var z_switch = z_front + 500;


    this.backZ   = z_back;
    this.switchZ = z_switch;
    this.shadowOffset = 5;
    frontPanel.switcher = this;
    backPanel.switcher = this;
    
    this.count = 0;
    this.steps = 12;
    this.steps = 16;
    this.overlayStep = 10;
    this.duration = 33;
    this.timer = null;
    this.running = false;

    this.startRad = Math.PI * 0.090;//Math.PI;
    this.endRad   = -Math.PI * .65;
    this.radius   = 70;
    this.frontCenter = 80;
    this.backCenter  = 120;

    this.frontStartPosition = { x: 0, y: 0 };
    this.backStartPosition = { x: 0, y: 0 };
    this.frontEndPosition = { x: 0, y: 0 };
    this.backEndPosition = { x: 0, y: 0 };

    this.setRadius = function( start, end ) {
        this.starRad = start;
        this.endRad = end;
        this.updatePosition();
    }
    this.setCenter = function( fcenter, bcenter ) {
        this.frontCenter = fcenter;
        this.backCenter = bcenter;
        this.updatePosition();
    }

    this.updatePosition = __PanelSwitcher__updatePosition;
    this.isRunning = function() { return this.running; };
    this.startTransition = __PanelSwitcher__startTransition;
    this.stopTransition  = __PanelSwitcher__stopTransition;
    this.tick            = __PanelSwitcher__tick;
    this.locked = false;
    this.isLocked = function() { return this.locked; };
    this.direction = 1; // front -> back

    this.endHandler = null;

    this.toString = function() {
        var output = "PanelSwitcher: " + this.steps + " steps, " + parseInt( this.startRad / Math.PI * 180 ) + " - " + parseInt( this.endRad / Math.PI * 180 );
        return output
    }

    // set initial positions
    this.updatePosition();
    this.frontPanel.style.top = parseInt( this.frontStartPosition.y + .5 ) + "px";
    this.frontShadow.style.top =parseInt( this.frontStartPosition.y + this.shadowOffset + .5 ) + "px";
    this.backPanel.style.top  = parseInt( this.backStartPosition.y + .5 ) + "px";
    debug( "FRONT : " + this.frontPanel.style.top + ", BACK : " + this.backPanel.style.top );

    // set close box
    if ( window.widget ) {
        widget.setCloseBoxOffset( this.frontStartPosition.x + 15, 
            this.frontStartPosition.y + 15 );
    }

}

function __PanelSwitcher__updatePosition() {
    this.frontStartPosition.y = Math.sin( this.startRad ) * this.radius + this.frontCenter;
    this.backStartPosition.y = Math.sin( this.startRad + Math.PI ) * this.radius + this.backCenter;
    this.frontEndPosition.y = Math.sin( this.endRad ) * this.radius + this.frontCenter;
    this.backEndPosition.y = Math.sin( this.endRad + Math.PI ) * this.radius + this.backCenter;

}

function __PanelSwitcher__startTransition( endHandler ) {
    if ( __PanelSwitcher_currentSwitcher != null || this.locked ) {
        return;
    }
    __PanelSwitcher_currentSwitcher = this;
    this.running = true;
    this.endHandler = endHandler;
    this.timer = setInterval( _PanelSwitcher__tick, this.duration );
    this.count = 0;
    _PanelSwitcher__tick();
}

function __PanelSwitcher__stopTransition() {
    __PanelSwitcher_currentSwitcher = null;
    this.running = false;
            // set close box
    if ( window.widget ) {
        if ( this.direction < 0 ) {
            widget.setCloseBoxOffset( this.frontStartPosition.x + 15, 
                                      this.frontStartPosition.y + 15 );
        } else if ( this.direction > 0 ) {
            widget.setCloseBoxOffset( this.frontEndPosition.x + 15, 
                                      this.frontEndPosition.y + 15 );
        }

    } 
    this.direction = - this.direction;
    clearInterval( this.timer );
    this.timer = null;
    if ( this.endHandler ) {
        this.endHandler( this );
        this.endHandler = null;
    }
}

function __PanelSwitcher__tick() {
    //
    var rad;
    if ( this.direction > 0 ) {
        if ( this.count == this.overlayStep ) {
            this.backPanel.style.zIndex = this.switchZ;
            //debug( "zIndex : " + this.switchZ );
        }
        rad = ( this.endRad - this.startRad ) / this.steps * ( this.count + 1 );
    } else {
        if ( this.count + this.overlayStep == this.steps ) {
            this.backPanel.style.zIndex = this.backZ;
            //debug( "zIndex : " + this.frontZ );
        }
        rad = this.endRad - ( this.endRad - this.startRad ) / this.steps * ( this.count + 1 );
    }
    var fy = parseInt( Math.sin( rad ) * this.radius + this.frontCenter  + .5 );
    var by = parseInt( Math.sin( rad + Math.PI ) * this.radius + this.backCenter + .5 );
    this.frontPanel.style.top = fy + "px";
    this.backPanel.style.top = by + "px";
    if ( this.frontShadow ) {
        this.frontShadow.style.top = ( fy + this.shadowOffset ) + "px";
    }

    this.count ++;
    if ( this.count >= this.steps ) {
        this.stopTransition();
    }
}

function _PanelSwitcher__tick() {
    if ( __PanelSwitcher_currentSwitcher != null ) {
        __PanelSwitcher_currentSwitcher.tick();
    }
}

